import boto3

from django.conf import settings
from django.test import TestCase

from io import BytesIO

from moto import mock_s3

from django_aws_s3_storage.settings import app_settings
from django_aws_s3_storage.storage import S3Storage


def create_s3_context():
    conn = boto3.resource('s3')
    conn.create_bucket(Bucket=app_settings.BUCKET_NAME)
    return conn


class ApplicationSettingsTestCase(TestCase):

    def test_application_must_be_well_configured(self):
        self.assertIn('django_aws_s3_storage', settings.INSTALLED_APPS)
        self.assertTrue(hasattr(settings, 'DJANGO_AWS_S3_STORAGE'))


class StorageTestCase(TestCase):
    storage = S3Storage()

    @mock_s3
    def test_uploaded_file_must_exist_on_bucket(self):
        create_s3_context()
        filename = 'random_file.txt'
        self.storage.save(filename, BytesIO())
        self.assertTrue(self.storage.exists(filename))



from django.conf import settings
from django.utils.translation import gettext as _


DEFAULT_SETTINGS = {
    'AWS_ACCESS_KEY_ID': None,
    'AWS_SECRET_ACCESS_KEY': None,
    'BUCKET_NAME': None,
    'MAX_AGE_SECONDS': 60,
    'PUBLIC_URL': '',
}

NONE_SETTING_NULL_ERROR_MESSAGE = '{} setting can not be None.'

# If `None` DjangoAWSS3StorageSettings.init was never called
app_settings = None


class DjangoAWSS3StorageSettings(object):

    def _setup(self):
        app_settings = getattr(settings, 'DJANGO_AWS_S3_STORAGE', None)
        if app_settings is None:
            raise NameError(_(NONE_SETTING_NULL_ERROR_MESSAGE.format('DJANGO_AWS_S3_STORAGE')))

        for k, default_v in DEFAULT_SETTINGS.items():
            v = app_settings.get(k, default_v)
            if v is None:
                raise NameError(_(NONE_SETTING_NULL_ERROR_MESSAGE.format(f'DJANGO_AWS_S3_STORAGE.{k}')))
            setattr(self, k, v)

    def __init__(self):
        self._setup()

    @staticmethod
    def init():
        global app_settings
        app_settings = DjangoAWSS3StorageSettings()



import boto3

from botocore.exceptions import ClientError

from io import BytesIO

from django.core.files.storage import Storage
from django.utils.deconstruct import deconstructible

from django_aws_s3_storage.settings import app_settings
from django_aws_s3_storage.utils import create_file_public_url


@deconstructible
class S3Storage(Storage):

    def __init__(self):
        self.client = boto3.client(
            's3',
            aws_access_key_id=app_settings.AWS_ACCESS_KEY_ID,
            aws_secret_access_key=app_settings.AWS_SECRET_ACCESS_KEY,
        )

    def _open(self, name, mode='rb'):
        obj = self.client.get_object(
            Bucket=app_settings.BUCKET_NAME,
            Key=name,
        )
        return obj['Body']

    def _save(self, name, content):
        # Always change the stream position to the beginning of the file
        content.seek(0)
        # Save the file in memory temporarily
        temp_file = BytesIO()

        for chunk in content.chunks():
            temp_file.write(chunk)

        self.client.put_object(
            Bucket=app_settings.BUCKET_NAME,
            Body=temp_file.getvalue(),
            Key=name,
            ContentType=content.content_type,
        )
        # Close the buffer of the file stored in memory
        temp_file.close()
        return name

    def delete(self, name):
        self.client.delete_object(Bucket=app_settings.BUCKET_NAME, Key=name)
        return name

    def exists(self, name):
        try:
            self.client.head_object(Bucket=app_settings.BUCKET_NAME, Key=name)
        except ClientError:
            return False
        return True

    def url(self, name):
        if app_settings.PUBLIC_URL != '':
            return create_file_public_url(app_settings.PUBLIC_URL, name)
        return self.client.generate_presigned_url(
            'get_object',
            Params={
                'Bucket': app_settings.BUCKET_NAME,
                'Key': name,
            },
            ExpiresIn=app_settings.MAX_AGE_SECONDS,
        )



from django.apps import AppConfig

from django_aws_s3_storage.settings import DjangoAWSS3StorageSettings


class DjangoAWSS3StorageConfig(AppConfig):
    name = 'django_aws_s3_storage'

    def ready(self):
        DjangoAWSS3StorageSettings.init()



from pathlib import Path

from setuptools import setup, find_packages


ROOT_DIR = Path(__file__).parent.resolve()


with open(ROOT_DIR.joinpath('requirements.txt')) as reqs_file:
    requirements = reqs_file.readlines()

with open(ROOT_DIR.joinpath('README.md')) as desc_file:
    long_description = desc_file.read()

setup(
    name='django-aws-s3-storage',
    version='0.1.0',
    description='AWS S3 Bucket file storage system for Django applications',
    long_description=long_description,
    long_description_content_type='text/markdown',
    author='Gustavo Valentim',
    packages=find_packages(where='src'),
    package_dir={'': 'src'},
    python_requires='>=3.6',
    install_requires=requirements,
)
